<?php

declare(strict_types=1);

namespace apexl\Io\modules\component\services;

use apexl\Io\modules\display\includes\region;

class regionBucket extends bucket
{
    public function set(string $name, region $value): void
    {
        parent::_set($name, $value);
    }

    public function get(string $name): ?region
    {
        return parent::_get($name);
    }
}
