<?php

namespace apexl\Io\modules\component;

use apexl\Io\includes\Module;

/**
 * Base module file, provides required methods to register things like routes and middleware.
 * Class installModule
 * @package apexl\Io\modules\install
 */
class componentModule extends Module
{
}
